# Copyrights 2013 by [Mark Overmeer].
#  For other contributors see Changes.
# See the manual pages for details on the licensing terms.
# Pod stripped from pm file by OODoc 2.01.
package Net::OAuth2;
use vars '$VERSION';
$VERSION = '0.54';


use warnings;
use strict;


1;
